/*----------------------------------------------------------------------- 
 * 
 * File Name: ArrayFactoriesTest.c
 * 
 * Revision: $Id: ArrayFactoriesTest.m4,v 1.1 2000/04/17 23:21:03 jolien Exp $
 * 
 *----------------------------------------------------------------------- 
 * 
 * DESCRIPTION 
 *   Test suite for vector factories
 * 
 * DIAGNOSTICS
 *   Writes PASS, FAIL to stdout as tests are passed or failed. 
 *   Returns 0 if all tests passed and -1 if any test failed. 
 * 
 * CALLS
 *   TYPECODECreateArray
 *   TYPECODEDestroyArray
 * 
 * NOTES
 *   PROGNAME can't test LALMalloc failure without messing with
 *   LALMalloc
 * 
 *-----------------------------------------------------------------------
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "LALConfig.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include "LALStdlib.h"
#include "AVFactories.h"

#define CODES_(x) #x
#define CODES(x) CODES_(x)

NRCSID( MAIN, "$Id: ArrayFactoriesTest.m4,v 1.1 2000/04/17 23:21:03 jolien Exp $" );

extern char *optarg;
extern int   optind;

int debuglevel = 1;
int verbose    = 0;

static void
Usage (const char *program, int exitflag);

static void
ParseOptions (int argc, char *argv[]);

static void
TestStatus (Status *status, const char *expectedCodes, int exitCode);

static void
ClearStatus (Status *status);



















void ZArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static COMPLEX16Array  *array;
  static COMPLEX16Array   astore;
  static COMPLEX16    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  ZCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( COMPLEX16 ) );

  ZDestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  ZCreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  ZCreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  ZCreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  ZDestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  ZCreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  ZDestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  ZCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  ZDestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  ZDestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of ZCreateArray and ZDestroyArray \n" );
          
  return;
}




















void CArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static COMPLEX8Array  *array;
  static COMPLEX8Array   astore;
  static COMPLEX8    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  CCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( COMPLEX8 ) );

  CDestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  CCreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  CCreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  CCreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  CDestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  CCreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  CDestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  CCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  CDestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  CDestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of CCreateArray and CDestroyArray \n" );
          
  return;
}




















void DArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static REAL8Array  *array;
  static REAL8Array   astore;
  static REAL8    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  DCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( REAL8 ) );

  DDestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  DCreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  DCreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  DCreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  DDestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  DCreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  DDestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  DCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  DDestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  DDestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of DCreateArray and DDestroyArray \n" );
          
  return;
}




















void SArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static REAL4Array  *array;
  static REAL4Array   astore;
  static REAL4    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  SCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( REAL4 ) );

  SDestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  SCreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  SCreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  SCreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  SDestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  SCreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  SDestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  SCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  SDestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  SDestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of SCreateArray and SDestroyArray \n" );
          
  return;
}




















void I2ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static INT2Array  *array;
  static INT2Array   astore;
  static INT2    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  I2CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( INT2 ) );

  I2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  I2CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  I2CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  I2CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  I2DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  I2CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  I2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  I2CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  I2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  I2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of I2CreateArray and I2DestroyArray \n" );
          
  return;
}




















void I4ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static INT4Array  *array;
  static INT4Array   astore;
  static INT4    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  I4CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( INT4 ) );

  I4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  I4CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  I4CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  I4CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  I4DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  I4CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  I4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  I4CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  I4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  I4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of I4CreateArray and I4DestroyArray \n" );
          
  return;
}




















void I8ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static INT8Array  *array;
  static INT8Array   astore;
  static INT8    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  I8CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( INT8 ) );

  I8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  I8CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  I8CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  I8CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  I8DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  I8CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  I8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  I8CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  I8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  I8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of I8CreateArray and I8DestroyArray \n" );
          
  return;
}




















void U2ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static UINT2Array  *array;
  static UINT2Array   astore;
  static UINT2    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  U2CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( UINT2 ) );

  U2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  U2CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  U2CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  U2CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  U2DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  U2CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  U2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  U2CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  U2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  U2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of U2CreateArray and U2DestroyArray \n" );
          
  return;
}




















void U4ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static UINT4Array  *array;
  static UINT4Array   astore;
  static UINT4    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  U4CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( UINT4 ) );

  U4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  U4CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  U4CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  U4CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  U4DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  U4CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  U4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  U4CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  U4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  U4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of U4CreateArray and U4DestroyArray \n" );
          
  return;
}




















void U8ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static UINT8Array  *array;
  static UINT8Array   astore;
  static UINT8    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  U8CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( UINT8 ) );

  U8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  U8CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  U8CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  U8CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  U8DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  U8CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  U8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  U8CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  U8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  U8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of U8CreateArray and U8DestroyArray \n" );
          
  return;
}




















void ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static Status  status;
  static REAL4Array  *array;
  static REAL4Array   astore;
  static REAL4    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( REAL4 ) );

  DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );

  CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( CREATEARRAY_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( DESTROYARRAY_EVPTR ), 1 );

  CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EVPTR ), 1 );

  DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EUPTR ), 1 );

  array = &astore;
  CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( CREATEARRAY_EUPTR ), 1 );

  DestroyArray ( &status, &array );
  TestStatus( &status, CODES( DESTROYARRAY_EDPTR ), 1 );

  array->data = &datum;
  DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of CreateArray and DestroyArray \n" );
          
  return;
}


int main( int argc, char *argv[] )
{
  ParseOptions( argc, argv );

  ArrayFactoriesTest();
  ZArrayFactoriesTest();
  CArrayFactoriesTest();
  DArrayFactoriesTest();
  SArrayFactoriesTest();
  I2ArrayFactoriesTest();
  I4ArrayFactoriesTest();
  I8ArrayFactoriesTest();
  U2ArrayFactoriesTest();
  U4ArrayFactoriesTest();
  U8ArrayFactoriesTest();

  return 0;
}


/*
 * TestStatus ()
 *
 * Routine to check that the status code status->statusCode agrees with one of
 * the codes specified in the space-delimited string ignored; if not,
 * exit to the system with code exitcode.
 *
 */
static void
TestStatus (Status *status, const char *ignored, int exitcode)
{
  char  str[64];
  char *tok;

  if (verbose)
  {
    REPORTSTATUS (status);
  }

  if (strncpy (str, ignored, sizeof (str)))
  {
    if ((tok = strtok (str, " ")))
    {
      do
      {
        if (status->statusCode == atoi (tok))
        {
          return;
        }
      }
      while ((tok = strtok (NULL, " ")));
    }
    else
    {
      if (status->statusCode == atoi (tok))
      {
        return;
      }
    }
  }

  fprintf (stderr, "\nExiting to system with code %d\n", exitcode);
  exit (exitcode);
}


/*
 *
 * ClearStatus ()
 *
 * Recursively applies DETATCHSTATUSPTR() to status structure to destroy
 * linked list of statuses.
 *
 */
void
ClearStatus (Status *status)
{
  if (status->statusPtr)
  {
    ClearStatus      (status->statusPtr);
    DETATCHSTATUSPTR (status);
  }
}


/*
 * Usage ()
 *
 * Prints a usage message for program program and exits with code exitcode.
 *
 */
static void
Usage (const char *program, int exitcode)
{
  fprintf (stderr, "Usage: %s [options]\n", program);
  fprintf (stderr, "Options:\n");
  fprintf (stderr, "  -h         print this message\n");
  fprintf (stderr, "  -q         quiet: run silently\n");
  fprintf (stderr, "  -v         verbose: print extra information\n");
  fprintf (stderr, "  -d level   set debuglevel to level\n");
  exit (exitcode);
}


/*
 * ParseOptions ()
 *
 * Parses the argc - 1 option strings in argv[].
 *
 */
static void
ParseOptions (int argc, char *argv[])
{
  while (1)
  {
    int c = -1;

    c = getopt (argc, argv, "hqvd:");
    if (c == -1)
    {
      break;
    }

    switch (c)
    {
      case 'd': /* set debug level */
        debuglevel = atoi (optarg);
        break;

      case 'v': /* verbose */
        ++verbose;
        break;

      case 'q': /* quiet: run silently (ignore error messages) */
        freopen ("/dev/null", "w", stderr);
        freopen ("/dev/null", "w", stdout);
        break;

      case 'h':
        Usage (argv[0], 0);
        break;

      default:
        Usage (argv[0], 1);
    }

  }

  if (optind < argc)
  {
    Usage (argv[0], 1);
  }

  return;
}

